package com.ruoyi.web.yh.report.service.impl;

import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.common.utils.collection.CollectionUtils;
import com.ruoyi.common.utils.collection.DiffListVO;
import com.ruoyi.web.yh.report.domain.bo.ReportBO;
import com.ruoyi.web.yh.reportProd.domain.ReportProd;
import com.ruoyi.web.yh.reportProd.domain.ReportProdVO;
import com.ruoyi.web.yh.reportProd.domain.bo.ReportProdBO;
import com.ruoyi.web.yh.reportProd.service.ReportProdService;
import com.ruoyi.web.yh.reportUserProd.domain.ReportUserProd;
import com.ruoyi.web.yh.reportUserProd.domain.ReportUserProdVO;
import com.ruoyi.web.yh.reportUserProd.domain.bo.ReportUserProdBO;
import com.ruoyi.web.yh.reportUserProd.service.ReportUserProdService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.web.yh.report.mapper.ReportMapper;
import com.ruoyi.web.yh.report.domain.Report;
import com.ruoyi.web.yh.report.domain.ReportVO;
import com.ruoyi.web.yh.report.domain.ReportQuery;
import com.ruoyi.web.yh.report.service.ReportService;
import org.springframework.transaction.support.TransactionTemplate;

/**
 * 报表Service业务层处理
 *
 * @author ruoyi
 * @date 2024-10-18
 */
@Service
public class ReportServiceImpl implements ReportService
{
    @Autowired
    private ReportMapper reportMapper;

    @Autowired
    private TransactionTemplate transactionTemplate;

    @Autowired
    private ReportProdService reportProdService;

    @Autowired
    private ReportUserProdService reportUserProdService;

    /**
     * 查询报表
     *
     * @param reportId 报表主键
     * @return 报表
     */
    @Override
    public ReportVO selectReportByReportId(Long reportId)
    {
        return reportMapper.selectReportByReportId(reportId);
    }

    /**
     * 查询报表列表
     *
     * @param report 报表
     * @return 报表
     */
    @Override
    public List<ReportVO> selectReportList(ReportQuery report)
    {
        return reportMapper.selectReportList(report);
    }

    /**
     * 新增报表
     *
     * @param report 报表
     * @return 结果
     */
    @Override
    public int insertReport(Report report)
    {
        report.setCreateTime(DateUtils.getNowDate());
        return reportMapper.insertReport(report);
    }

    /**
     * 修改报表
     *
     * @param report 报表
     * @return 结果
     */
    @Override
    public int updateReport(Report report)
    {
        report.setUpdateTime(DateUtils.getNowDate());
        return reportMapper.updateReport(report);
    }

    /**
     * 批量删除报表
     *
     * @param reportIds 需要删除的报表主键
     * @return 结果
     */
    @Override
    public int deleteReportByReportIds(Long[] reportIds)
    {
        return reportMapper.deleteReportByReportIds(reportIds);
    }

    /**
     * 删除报表信息
     *
     * @param reportId 报表主键
     * @return 结果
     */
    @Override
    public int deleteReportByReportId(Long reportId)
    {
        return reportMapper.deleteReportByReportId(reportId);
    }

    @Override
    public int addReport(ReportBO bo) {
        ServiceUtil.assertion(bo == null, "参数错误");
        // TODO 校验

        Integer result = transactionTemplate.execute(status -> {
            // 新增主表
            int insert = this.insertReport(bo);
            ServiceUtil.assertion(insert != 1, "新增报表失败");

            // 更新产量表
            this.batchUpdateProductList(bo);

            // 更新员工产量表
            this.batchUpdateUserProductList(bo);

            return insert;
        });

        return result == null ? 0 : result;
    }

    /**
     * 更新员工产量表
     * @param bo
     * @return
     */
    private int batchUpdateUserProductList(ReportBO bo) {
        if (bo == null || bo.getReportId() == null || CollectionUtils.isEmptyElement(bo.getProductList())) {
            return 0;
        }
        // 设置总产量ID
        for (ReportProdBO prod : bo.getProductList()) {
            if (prod.getUserProdList() == null) {
                continue;
            }
            for (ReportUserProdBO userProd : prod.getUserProdList()) {
                userProd.setProdId(prod.getId());
            }
        }

        // 扁平化，获取列表
        List<ReportUserProdBO> newList = bo.getProductList().stream()
                .map(ReportProdBO::getUserProdList)
                .flatMap(List::stream)
                .filter(Objects::nonNull)
                .collect(Collectors.toList());

        // 查询旧列表
        List<ReportUserProdVO> oldList = reportUserProdService.selectListByReportId(bo.getReportId());

        // 获取差异
        DiffListVO<ReportUserProdBO, ReportUserProdVO> diff = CollectionUtils.convertToDiffList(newList, oldList, ReportUserProd::getId);

        int result = 0;
        if (diff.getAddCount() > 0) {
            int insert = reportUserProdService.batchInsert(diff.getAddList());
            ServiceUtil.assertion(insert != diff.getAddCount(), "新增用户产量明细失败");
            result += insert;
        }
        if (diff.getUpdateCount() > 0) {
            int update = reportUserProdService.batchUpdate(diff.getUpdateList());
            ServiceUtil.assertion(update != diff.getUpdateCount(), "修改用户产量明细失败");
            result += update;
        }
        if (diff.getDelCount() > 0) {
            int del = reportUserProdService.batchLogicDel(diff.getDelList());
            ServiceUtil.assertion(del != diff.getDelCount(), "删除用户产量明细失败");
            result += del;
        }

        return result;
    }

    @Override
    public int editReport(ReportBO bo) {
        ServiceUtil.assertion(bo == null, "参数错误");

        Integer result = transactionTemplate.execute(status -> {
            // 修改主表
            int update = this.updateReport(bo);
            ServiceUtil.assertion(update != 1, "修改报表失败");

            // 修改产量表
            this.batchUpdateProductList(bo);

            // 修改员工产量表
            this.batchUpdateUserProductList(bo);

            return update;
        });

        return result == null ? 0 : result;
    }

    /**
     * 更新产量表
     */
    private int batchUpdateProductList(ReportBO bo) {
        if (bo == null || CollectionUtils.isEmptyElement(bo.getProductList())) {
            return 0;
        }
        // 修改报表ID
        for (ReportProd prod : bo.getProductList()) {
            prod.setReportId(bo.getReportId());
        }

        List<ReportProdVO> oldList = reportProdService.selectByReportId(bo.getReportId());

        // 分离出新增、修改、删除的列表
        DiffListVO<ReportProdBO, ReportProdVO> diff = CollectionUtils.convertToDiffList(bo.getProductList(), oldList, ReportProd::getId);

        int result = 0;
        if (diff.getAddCount() > 0) {
            int batchInsert = reportProdService.batchInsert(diff.getAddList());
            ServiceUtil.assertion(batchInsert != diff.getAddCount(), "新增报表产量明细失败");
            result += batchInsert;
        }
        if (diff.getUpdateCount() > 0) {
            int update = reportProdService.batchUpdate(diff.getUpdateList());
            ServiceUtil.assertion(update != diff.getUpdateCount(), "修改报表产量明细失败");
            result += update;
        }
        if (diff.getDelCount() > 0) {
            int del = reportProdService.batchLogicDel(diff.getDelList());
            ServiceUtil.assertion(del != diff.getDelCount(), "删除报表产量明细失败");
            result += del;
        }

        return result;
    }
}
